/* zeitgeist.h generated by valac 0.56.19, the Vala compiler, do not modify */

#ifndef __ZEITGEIST_H__
#define __ZEITGEIST_H__

#include <glib-object.h>
#include <gio/gio.h>
#include <glib.h>
#include <stdlib.h>
#include <string.h>
#include "zeitgeist-datamodel.h"
#include <float.h>
#include <math.h>

G_BEGIN_DECLS

#if !defined(VALA_EXTERN)
#if defined(_MSC_VER)
#define VALA_EXTERN __declspec(dllexport) extern
#elif __GNUC__ >= 4
#define VALA_EXTERN __attribute__((visibility("default"))) extern
#else
#define VALA_EXTERN extern
#endif
#endif

#define ZEITGEIST_TYPE_REMOTE_REGISTRY (zeitgeist_remote_registry_get_type ())
#define ZEITGEIST_REMOTE_REGISTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_REMOTE_REGISTRY, ZeitgeistRemoteRegistry))
#define ZEITGEIST_IS_REMOTE_REGISTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_REMOTE_REGISTRY))
#define ZEITGEIST_REMOTE_REGISTRY_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ZEITGEIST_TYPE_REMOTE_REGISTRY, ZeitgeistRemoteRegistryIface))

typedef struct _ZeitgeistRemoteRegistry ZeitgeistRemoteRegistry;
typedef struct _ZeitgeistRemoteRegistryIface ZeitgeistRemoteRegistryIface;

#define ZEITGEIST_TYPE_REMOTE_REGISTRY_PROXY (zeitgeist_remote_registry_proxy_get_type ())

#define ZEITGEIST_TYPE_QUEUED_PROXY_WRAPPER (zeitgeist_queued_proxy_wrapper_get_type ())
#define ZEITGEIST_QUEUED_PROXY_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_QUEUED_PROXY_WRAPPER, ZeitgeistQueuedProxyWrapper))
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ZEITGEIST_TYPE_QUEUED_PROXY_WRAPPER, ZeitgeistQueuedProxyWrapperClass))
#define ZEITGEIST_IS_QUEUED_PROXY_WRAPPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_QUEUED_PROXY_WRAPPER))
#define ZEITGEIST_IS_QUEUED_PROXY_WRAPPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ZEITGEIST_TYPE_QUEUED_PROXY_WRAPPER))
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ZEITGEIST_TYPE_QUEUED_PROXY_WRAPPER, ZeitgeistQueuedProxyWrapperClass))

typedef struct _ZeitgeistQueuedProxyWrapper ZeitgeistQueuedProxyWrapper;
typedef struct _ZeitgeistQueuedProxyWrapperClass ZeitgeistQueuedProxyWrapperClass;
typedef struct _ZeitgeistQueuedProxyWrapperPrivate ZeitgeistQueuedProxyWrapperPrivate;

#define ZEITGEIST_TYPE_DATA_SOURCE_REGISTRY (zeitgeist_data_source_registry_get_type ())
#define ZEITGEIST_DATA_SOURCE_REGISTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_DATA_SOURCE_REGISTRY, ZeitgeistDataSourceRegistry))
#define ZEITGEIST_DATA_SOURCE_REGISTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ZEITGEIST_TYPE_DATA_SOURCE_REGISTRY, ZeitgeistDataSourceRegistryClass))
#define ZEITGEIST_IS_DATA_SOURCE_REGISTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_DATA_SOURCE_REGISTRY))
#define ZEITGEIST_IS_DATA_SOURCE_REGISTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ZEITGEIST_TYPE_DATA_SOURCE_REGISTRY))
#define ZEITGEIST_DATA_SOURCE_REGISTRY_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ZEITGEIST_TYPE_DATA_SOURCE_REGISTRY, ZeitgeistDataSourceRegistryClass))

typedef struct _ZeitgeistDataSourceRegistry ZeitgeistDataSourceRegistry;
typedef struct _ZeitgeistDataSourceRegistryClass ZeitgeistDataSourceRegistryClass;
typedef struct _ZeitgeistDataSourceRegistryPrivate ZeitgeistDataSourceRegistryPrivate;

#define ZEITGEIST_TYPE_INDEX (zeitgeist_index_get_type ())
#define ZEITGEIST_INDEX(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_INDEX, ZeitgeistIndex))
#define ZEITGEIST_INDEX_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ZEITGEIST_TYPE_INDEX, ZeitgeistIndexClass))
#define ZEITGEIST_IS_INDEX(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_INDEX))
#define ZEITGEIST_IS_INDEX_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ZEITGEIST_TYPE_INDEX))
#define ZEITGEIST_INDEX_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ZEITGEIST_TYPE_INDEX, ZeitgeistIndexClass))

typedef struct _ZeitgeistIndex ZeitgeistIndex;
typedef struct _ZeitgeistIndexClass ZeitgeistIndexClass;
typedef struct _ZeitgeistIndexPrivate ZeitgeistIndexPrivate;

#define ZEITGEIST_TYPE_LOG (zeitgeist_log_get_type ())
#define ZEITGEIST_LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_LOG, ZeitgeistLog))
#define ZEITGEIST_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ZEITGEIST_TYPE_LOG, ZeitgeistLogClass))
#define ZEITGEIST_IS_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_LOG))
#define ZEITGEIST_IS_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ZEITGEIST_TYPE_LOG))
#define ZEITGEIST_LOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ZEITGEIST_TYPE_LOG, ZeitgeistLogClass))

typedef struct _ZeitgeistLog ZeitgeistLog;
typedef struct _ZeitgeistLogClass ZeitgeistLogClass;
typedef struct _ZeitgeistLogPrivate ZeitgeistLogPrivate;

#define ZEITGEIST_TYPE_MONITOR (zeitgeist_monitor_get_type ())
#define ZEITGEIST_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_MONITOR, ZeitgeistMonitor))
#define ZEITGEIST_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ZEITGEIST_TYPE_MONITOR, ZeitgeistMonitorClass))
#define ZEITGEIST_IS_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_MONITOR))
#define ZEITGEIST_IS_MONITOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ZEITGEIST_TYPE_MONITOR))
#define ZEITGEIST_MONITOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ZEITGEIST_TYPE_MONITOR, ZeitgeistMonitorClass))

typedef struct _ZeitgeistMonitor ZeitgeistMonitor;
typedef struct _ZeitgeistMonitorClass ZeitgeistMonitorClass;

#define ZEITGEIST_TYPE_REMOTE_MONITOR (zeitgeist_remote_monitor_get_type ())
#define ZEITGEIST_REMOTE_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_REMOTE_MONITOR, ZeitgeistRemoteMonitor))
#define ZEITGEIST_IS_REMOTE_MONITOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_REMOTE_MONITOR))
#define ZEITGEIST_REMOTE_MONITOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ZEITGEIST_TYPE_REMOTE_MONITOR, ZeitgeistRemoteMonitorIface))

typedef struct _ZeitgeistRemoteMonitor ZeitgeistRemoteMonitor;
typedef struct _ZeitgeistRemoteMonitorIface ZeitgeistRemoteMonitorIface;

#define ZEITGEIST_TYPE_REMOTE_MONITOR_PROXY (zeitgeist_remote_monitor_proxy_get_type ())
typedef struct _ZeitgeistMonitorPrivate ZeitgeistMonitorPrivate;

#define ZEITGEIST_QUEUED_PROXY_WRAPPER_TYPE_QUEUED_METHOD (zeitgeist_queued_proxy_wrapper_queued_method_get_type ())
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_QUEUED_METHOD(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_QUEUED_PROXY_WRAPPER_TYPE_QUEUED_METHOD, ZeitgeistQueuedProxyWrapperQueuedMethod))
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_QUEUED_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), ZEITGEIST_QUEUED_PROXY_WRAPPER_TYPE_QUEUED_METHOD, ZeitgeistQueuedProxyWrapperQueuedMethodClass))
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_IS_QUEUED_METHOD(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_QUEUED_PROXY_WRAPPER_TYPE_QUEUED_METHOD))
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_IS_QUEUED_METHOD_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), ZEITGEIST_QUEUED_PROXY_WRAPPER_TYPE_QUEUED_METHOD))
#define ZEITGEIST_QUEUED_PROXY_WRAPPER_QUEUED_METHOD_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), ZEITGEIST_QUEUED_PROXY_WRAPPER_TYPE_QUEUED_METHOD, ZeitgeistQueuedProxyWrapperQueuedMethodClass))

typedef struct _ZeitgeistQueuedProxyWrapperQueuedMethod ZeitgeistQueuedProxyWrapperQueuedMethod;
typedef struct _ZeitgeistQueuedProxyWrapperQueuedMethodClass ZeitgeistQueuedProxyWrapperQueuedMethodClass;
typedef struct _ZeitgeistQueuedProxyWrapperQueuedMethodPrivate ZeitgeistQueuedProxyWrapperQueuedMethodPrivate;

#define ZEITGEIST_TYPE_VERSION_STRUCT (zeitgeist_version_struct_get_type ())
typedef struct _ZeitgeistVersionStruct ZeitgeistVersionStruct;

#define ZEITGEIST_TYPE_REMOTE_LOG (zeitgeist_remote_log_get_type ())
#define ZEITGEIST_REMOTE_LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_REMOTE_LOG, ZeitgeistRemoteLog))
#define ZEITGEIST_IS_REMOTE_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_REMOTE_LOG))
#define ZEITGEIST_REMOTE_LOG_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ZEITGEIST_TYPE_REMOTE_LOG, ZeitgeistRemoteLogIface))

typedef struct _ZeitgeistRemoteLog ZeitgeistRemoteLog;
typedef struct _ZeitgeistRemoteLogIface ZeitgeistRemoteLogIface;

#define ZEITGEIST_TYPE_REMOTE_LOG_PROXY (zeitgeist_remote_log_proxy_get_type ())

#define ZEITGEIST_TYPE_REMOTE_SIMPLE_INDEXER (zeitgeist_remote_simple_indexer_get_type ())
#define ZEITGEIST_REMOTE_SIMPLE_INDEXER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_REMOTE_SIMPLE_INDEXER, ZeitgeistRemoteSimpleIndexer))
#define ZEITGEIST_IS_REMOTE_SIMPLE_INDEXER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_REMOTE_SIMPLE_INDEXER))
#define ZEITGEIST_REMOTE_SIMPLE_INDEXER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ZEITGEIST_TYPE_REMOTE_SIMPLE_INDEXER, ZeitgeistRemoteSimpleIndexerIface))

typedef struct _ZeitgeistRemoteSimpleIndexer ZeitgeistRemoteSimpleIndexer;
typedef struct _ZeitgeistRemoteSimpleIndexerIface ZeitgeistRemoteSimpleIndexerIface;

#define ZEITGEIST_TYPE_REMOTE_SIMPLE_INDEXER_PROXY (zeitgeist_remote_simple_indexer_proxy_get_type ())

#define ZEITGEIST_TYPE_NETWORK_MANAGER_DBUS (zeitgeist_network_manager_dbus_get_type ())
#define ZEITGEIST_NETWORK_MANAGER_DBUS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_NETWORK_MANAGER_DBUS, ZeitgeistNetworkManagerDBus))
#define ZEITGEIST_IS_NETWORK_MANAGER_DBUS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_NETWORK_MANAGER_DBUS))
#define ZEITGEIST_NETWORK_MANAGER_DBUS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ZEITGEIST_TYPE_NETWORK_MANAGER_DBUS, ZeitgeistNetworkManagerDBusIface))

typedef struct _ZeitgeistNetworkManagerDBus ZeitgeistNetworkManagerDBus;
typedef struct _ZeitgeistNetworkManagerDBusIface ZeitgeistNetworkManagerDBusIface;

#define ZEITGEIST_TYPE_NETWORK_MANAGER_DBUS_PROXY (zeitgeist_network_manager_dbus_proxy_get_type ())

#define ZEITGEIST_TYPE_CONNMAN_MANAGER_DBUS (zeitgeist_connman_manager_dbus_get_type ())
#define ZEITGEIST_CONNMAN_MANAGER_DBUS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), ZEITGEIST_TYPE_CONNMAN_MANAGER_DBUS, ZeitgeistConnmanManagerDBus))
#define ZEITGEIST_IS_CONNMAN_MANAGER_DBUS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), ZEITGEIST_TYPE_CONNMAN_MANAGER_DBUS))
#define ZEITGEIST_CONNMAN_MANAGER_DBUS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), ZEITGEIST_TYPE_CONNMAN_MANAGER_DBUS, ZeitgeistConnmanManagerDBusIface))

typedef struct _ZeitgeistConnmanManagerDBus ZeitgeistConnmanManagerDBus;
typedef struct _ZeitgeistConnmanManagerDBusIface ZeitgeistConnmanManagerDBusIface;

#define ZEITGEIST_TYPE_CONNMAN_MANAGER_DBUS_PROXY (zeitgeist_connman_manager_dbus_proxy_get_type ())

struct _ZeitgeistRemoteRegistryIface {
	GTypeInterface parent_iface;
	void (*get_data_sources) (ZeitgeistRemoteRegistry* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GVariant* (*get_data_sources_finish) (ZeitgeistRemoteRegistry* self, GAsyncResult* _res_, GError** error);
	void (*register_data_source) (ZeitgeistRemoteRegistry* self, const gchar* unique_id, const gchar* name, const gchar* description, GVariant* event_templates, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gboolean (*register_data_source_finish) (ZeitgeistRemoteRegistry* self, GAsyncResult* _res_, GError** error);
	void (*set_data_source_enabled) (ZeitgeistRemoteRegistry* self, const gchar* unique_id, gboolean enabled, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*set_data_source_enabled_finish) (ZeitgeistRemoteRegistry* self, GAsyncResult* _res_, GError** error);
	void (*get_data_source_from_id) (ZeitgeistRemoteRegistry* self, const gchar* unique_id, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GVariant* (*get_data_source_from_id_finish) (ZeitgeistRemoteRegistry* self, GAsyncResult* _res_, GError** error);
};

struct _ZeitgeistQueuedProxyWrapper {
	GObject parent_instance;
	ZeitgeistQueuedProxyWrapperPrivate * priv;
};

struct _ZeitgeistQueuedProxyWrapperClass {
	GObjectClass parent_class;
	void (*on_connection_established) (ZeitgeistQueuedProxyWrapper* self);
	void (*on_connection_lost) (ZeitgeistQueuedProxyWrapper* self);
};

struct _ZeitgeistDataSourceRegistry {
	ZeitgeistQueuedProxyWrapper parent_instance;
	ZeitgeistDataSourceRegistryPrivate * priv;
};

struct _ZeitgeistDataSourceRegistryClass {
	ZeitgeistQueuedProxyWrapperClass parent_class;
};

struct _ZeitgeistIndex {
	ZeitgeistQueuedProxyWrapper parent_instance;
	ZeitgeistIndexPrivate * priv;
};

struct _ZeitgeistIndexClass {
	ZeitgeistQueuedProxyWrapperClass parent_class;
};

struct _ZeitgeistLog {
	ZeitgeistQueuedProxyWrapper parent_instance;
	ZeitgeistLogPrivate * priv;
};

struct _ZeitgeistLogClass {
	ZeitgeistQueuedProxyWrapperClass parent_class;
};

struct _ZeitgeistRemoteMonitorIface {
	GTypeInterface parent_iface;
	void (*notify_insert) (ZeitgeistRemoteMonitor* self, GVariant* time_range, GVariant* events, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*notify_insert_finish) (ZeitgeistRemoteMonitor* self, GAsyncResult* _res_, GError** error);
	void (*notify_delete) (ZeitgeistRemoteMonitor* self, GVariant* time_range, guint32* event_ids, gint event_ids_length1, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*notify_delete_finish) (ZeitgeistRemoteMonitor* self, GAsyncResult* _res_, GError** error);
};

struct _ZeitgeistMonitor {
	GObject parent_instance;
	ZeitgeistMonitorPrivate * priv;
};

struct _ZeitgeistMonitorClass {
	GObjectClass parent_class;
};

struct _ZeitgeistQueuedProxyWrapperQueuedMethod {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ZeitgeistQueuedProxyWrapperQueuedMethodPrivate * priv;
};

struct _ZeitgeistQueuedProxyWrapperQueuedMethodClass {
	GTypeClass parent_class;
	void (*finalize) (ZeitgeistQueuedProxyWrapperQueuedMethod *self);
};

struct _ZeitgeistVersionStruct {
	gint major;
	gint minor;
	gint micro;
};

struct _ZeitgeistRemoteLogIface {
	GTypeInterface parent_iface;
	void (*delete_events) (ZeitgeistRemoteLog* self, guint32* event_ids, gint event_ids_length1, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GVariant* (*delete_events_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, GError** error);
	void (*find_event_ids) (ZeitgeistRemoteLog* self, GVariant* time_range, GVariant* event_templates, guint storage_state, guint num_events, guint result_type, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	guint32* (*find_event_ids_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, gint* result_length1, GError** error);
	void (*find_events) (ZeitgeistRemoteLog* self, GVariant* time_range, GVariant* event_templates, guint storage_state, guint num_events, guint result_type, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GVariant* (*find_events_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, GError** error);
	void (*find_related_uris) (ZeitgeistRemoteLog* self, GVariant* time_range, GVariant* event_templates, GVariant* result_event_templates, guint storage_state, guint num_events, guint result_type, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	gchar** (*find_related_uris_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, gint* result_length1, GError** error);
	void (*get_events) (ZeitgeistRemoteLog* self, guint32* event_ids, gint event_ids_length1, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	GVariant* (*get_events_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, GError** error);
	void (*insert_events) (ZeitgeistRemoteLog* self, GVariant* events, GCancellable* cancellable, const char* sender, GAsyncReadyCallback _callback_, gpointer _user_data_);
	guint32* (*insert_events_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, gint* result_length1, GError** error);
	void (*install_monitor) (ZeitgeistRemoteLog* self, const char* monitor_path, GVariant* time_range, GVariant* event_templates, const char* owner, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*install_monitor_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, GError** error);
	void (*remove_monitor) (ZeitgeistRemoteLog* self, const char* monitor_path, const char* owner, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*remove_monitor_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, GError** error);
	void (*quit) (ZeitgeistRemoteLog* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*quit_finish) (ZeitgeistRemoteLog* self, GAsyncResult* _res_, GError** error);
	gchar** (*get_extensions) (ZeitgeistRemoteLog* self, gint* result_length1);
	void (*get_version) (ZeitgeistRemoteLog* self, ZeitgeistVersionStruct * result);
	gchar* (*get_datapath) (ZeitgeistRemoteLog* self);
};

struct _ZeitgeistRemoteSimpleIndexerIface {
	GTypeInterface parent_iface;
	void (*search) (ZeitgeistRemoteSimpleIndexer* self, const gchar* query_string, GVariant* time_range, GVariant* filter_templates, guint offset, guint count, guint result_type, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*search_finish) (ZeitgeistRemoteSimpleIndexer* self, GAsyncResult* _res_, GVariant** events, guint* matches, GError** error);
	void (*search_with_relevancies) (ZeitgeistRemoteSimpleIndexer* self, const gchar* query_string, GVariant* time_range, GVariant* filter_templates, guint storage_state, guint offset, guint count, guint result_type, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_);
	void (*search_with_relevancies_finish) (ZeitgeistRemoteSimpleIndexer* self, GAsyncResult* _res_, GVariant** events, gdouble** relevancies, gint* relevancies_length1, guint* matches, GError** error);
};

struct _ZeitgeistNetworkManagerDBusIface {
	GTypeInterface parent_iface;
	guint32 (*state) (ZeitgeistNetworkManagerDBus* self, GError** error);
};

struct _ZeitgeistConnmanManagerDBusIface {
	GTypeInterface parent_iface;
	gchar* (*get_state) (ZeitgeistConnmanManagerDBus* self, GError** error);
};

VALA_EXTERN GType zeitgeist_remote_registry_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint zeitgeist_remote_registry_register_object (void* object,
                                                 GDBusConnection* connection,
                                                 const gchar* path,
                                                 GError** error);
VALA_EXTERN GType zeitgeist_remote_registry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistRemoteRegistry, g_object_unref)
VALA_EXTERN void zeitgeist_remote_registry_get_data_sources (ZeitgeistRemoteRegistry* self,
                                                 GCancellable* cancellable,
                                                 GAsyncReadyCallback _callback_,
                                                 gpointer _user_data_);
VALA_EXTERN GVariant* zeitgeist_remote_registry_get_data_sources_finish (ZeitgeistRemoteRegistry* self,
                                                             GAsyncResult* _res_,
                                                             GError** error);
VALA_EXTERN void zeitgeist_remote_registry_register_data_source (ZeitgeistRemoteRegistry* self,
                                                     const gchar* unique_id,
                                                     const gchar* name,
                                                     const gchar* description,
                                                     GVariant* event_templates,
                                                     GCancellable* cancellable,
                                                     const char* sender,
                                                     GAsyncReadyCallback _callback_,
                                                     gpointer _user_data_);
VALA_EXTERN gboolean zeitgeist_remote_registry_register_data_source_finish (ZeitgeistRemoteRegistry* self,
                                                                GAsyncResult* _res_,
                                                                GError** error);
VALA_EXTERN void zeitgeist_remote_registry_set_data_source_enabled (ZeitgeistRemoteRegistry* self,
                                                        const gchar* unique_id,
                                                        gboolean enabled,
                                                        GCancellable* cancellable,
                                                        GAsyncReadyCallback _callback_,
                                                        gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_registry_set_data_source_enabled_finish (ZeitgeistRemoteRegistry* self,
                                                               GAsyncResult* _res_,
                                                               GError** error);
VALA_EXTERN void zeitgeist_remote_registry_get_data_source_from_id (ZeitgeistRemoteRegistry* self,
                                                        const gchar* unique_id,
                                                        GCancellable* cancellable,
                                                        GAsyncReadyCallback _callback_,
                                                        gpointer _user_data_);
VALA_EXTERN GVariant* zeitgeist_remote_registry_get_data_source_from_id_finish (ZeitgeistRemoteRegistry* self,
                                                                    GAsyncResult* _res_,
                                                                    GError** error);
VALA_EXTERN GType zeitgeist_queued_proxy_wrapper_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistQueuedProxyWrapper, g_object_unref)
VALA_EXTERN GType zeitgeist_data_source_registry_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistDataSourceRegistry, g_object_unref)
VALA_EXTERN ZeitgeistDataSourceRegistry* zeitgeist_data_source_registry_new (void);
VALA_EXTERN ZeitgeistDataSourceRegistry* zeitgeist_data_source_registry_construct (GType object_type);
VALA_EXTERN void zeitgeist_data_source_registry_get_data_sources (ZeitgeistDataSourceRegistry* self,
                                                      GCancellable* cancellable,
                                                      GAsyncReadyCallback _callback_,
                                                      gpointer _user_data_);
VALA_EXTERN GPtrArray* zeitgeist_data_source_registry_get_data_sources_finish (ZeitgeistDataSourceRegistry* self,
                                                                   GAsyncResult* _res_,
                                                                   GError** error);
VALA_EXTERN void zeitgeist_data_source_registry_get_data_source_from_id (ZeitgeistDataSourceRegistry* self,
                                                             const gchar* unique_id,
                                                             GCancellable* cancellable,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN ZeitgeistDataSource* zeitgeist_data_source_registry_get_data_source_from_id_finish (ZeitgeistDataSourceRegistry* self,
                                                                                    GAsyncResult* _res_,
                                                                                    GError** error);
VALA_EXTERN void zeitgeist_data_source_registry_register_data_source (ZeitgeistDataSourceRegistry* self,
                                                          ZeitgeistDataSource* data_source,
                                                          GCancellable* cancellable,
                                                          GAsyncReadyCallback _callback_,
                                                          gpointer _user_data_);
VALA_EXTERN gboolean zeitgeist_data_source_registry_register_data_source_finish (ZeitgeistDataSourceRegistry* self,
                                                                     GAsyncResult* _res_,
                                                                     GError** error);
VALA_EXTERN void zeitgeist_data_source_registry_set_data_source_enabled (ZeitgeistDataSourceRegistry* self,
                                                             const gchar* unique_id,
                                                             gboolean enabled,
                                                             GCancellable* cancellable,
                                                             GAsyncReadyCallback _callback_,
                                                             gpointer _user_data_);
VALA_EXTERN void zeitgeist_data_source_registry_set_data_source_enabled_finish (ZeitgeistDataSourceRegistry* self,
                                                                    GAsyncResult* _res_,
                                                                    GError** error);
VALA_EXTERN GType zeitgeist_index_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistIndex, g_object_unref)
VALA_EXTERN ZeitgeistIndex* zeitgeist_index_new (void);
VALA_EXTERN ZeitgeistIndex* zeitgeist_index_construct (GType object_type);
VALA_EXTERN void zeitgeist_index_search (ZeitgeistIndex* self,
                             const gchar* query,
                             ZeitgeistTimeRange* time_range,
                             GPtrArray* event_templates,
                             guint32 offset,
                             guint32 num_events,
                             ZeitgeistResultType result_type,
                             GCancellable* cancellable,
                             GAsyncReadyCallback _callback_,
                             gpointer _user_data_);
VALA_EXTERN ZeitgeistResultSet* zeitgeist_index_search_finish (ZeitgeistIndex* self,
                                                   GAsyncResult* _res_,
                                                   GError** error);
VALA_EXTERN void zeitgeist_index_search_with_relevancies (ZeitgeistIndex* self,
                                              const gchar* query,
                                              ZeitgeistTimeRange* time_range,
                                              GPtrArray* event_templates,
                                              ZeitgeistStorageState storage_state,
                                              guint32 offset,
                                              guint32 num_events,
                                              ZeitgeistResultType result_type,
                                              GCancellable* cancellable,
                                              GAsyncReadyCallback _callback_,
                                              gpointer _user_data_);
VALA_EXTERN ZeitgeistResultSet* zeitgeist_index_search_with_relevancies_finish (ZeitgeistIndex* self,
                                                                    GAsyncResult* _res_,
                                                                    gdouble** relevancies,
                                                                    gint* relevancies_length1,
                                                                    GError** error);
VALA_EXTERN GType zeitgeist_log_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistLog, g_object_unref)
VALA_EXTERN ZeitgeistLog* zeitgeist_log_new (void);
VALA_EXTERN ZeitgeistLog* zeitgeist_log_construct (GType object_type);
VALA_EXTERN ZeitgeistLog* zeitgeist_log_get_default (void);
VALA_EXTERN void zeitgeist_log_insert_event (ZeitgeistLog* self,
                                 ZeitgeistEvent* event,
                                 GCancellable* cancellable,
                                 GAsyncReadyCallback _callback_,
                                 gpointer _user_data_);
VALA_EXTERN GArray* zeitgeist_log_insert_event_finish (ZeitgeistLog* self,
                                           GAsyncResult* _res_,
                                           GError** error);
VALA_EXTERN void zeitgeist_log_insert_events (ZeitgeistLog* self,
                                  GPtrArray* events,
                                  GCancellable* cancellable,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN GArray* zeitgeist_log_insert_events_finish (ZeitgeistLog* self,
                                            GAsyncResult* _res_,
                                            GError** error);
VALA_EXTERN void zeitgeist_log_insert_event_no_reply (ZeitgeistLog* self,
                                          ZeitgeistEvent* event,
                                          GError** error);
VALA_EXTERN void zeitgeist_log_insert_events_no_reply (ZeitgeistLog* self,
                                           GPtrArray* events,
                                           GError** error);
VALA_EXTERN void zeitgeist_log_find_events (ZeitgeistLog* self,
                                ZeitgeistTimeRange* time_range,
                                GPtrArray* event_templates,
                                ZeitgeistStorageState storage_state,
                                guint32 num_events,
                                ZeitgeistResultType result_type,
                                GCancellable* cancellable,
                                GAsyncReadyCallback _callback_,
                                gpointer _user_data_);
VALA_EXTERN ZeitgeistResultSet* zeitgeist_log_find_events_finish (ZeitgeistLog* self,
                                                      GAsyncResult* _res_,
                                                      GError** error);
VALA_EXTERN void zeitgeist_log_find_event_ids (ZeitgeistLog* self,
                                   ZeitgeistTimeRange* time_range,
                                   GPtrArray* event_templates,
                                   ZeitgeistStorageState storage_state,
                                   guint32 num_events,
                                   ZeitgeistResultType result_type,
                                   GCancellable* cancellable,
                                   GAsyncReadyCallback _callback_,
                                   gpointer _user_data_);
VALA_EXTERN guint32* zeitgeist_log_find_event_ids_finish (ZeitgeistLog* self,
                                              GAsyncResult* _res_,
                                              gint* result_length1,
                                              GError** error);
VALA_EXTERN void zeitgeist_log_get_events (ZeitgeistLog* self,
                               GArray* event_ids,
                               GCancellable* cancellable,
                               GAsyncReadyCallback _callback_,
                               gpointer _user_data_);
VALA_EXTERN ZeitgeistResultSet* zeitgeist_log_get_events_finish (ZeitgeistLog* self,
                                                     GAsyncResult* _res_,
                                                     GError** error);
VALA_EXTERN void zeitgeist_log_find_related_uris (ZeitgeistLog* self,
                                      ZeitgeistTimeRange* time_range,
                                      GPtrArray* event_templates,
                                      GPtrArray* result_event_templates,
                                      ZeitgeistStorageState storage_state,
                                      guint32 num_events,
                                      ZeitgeistRelevantResultType result_type,
                                      GCancellable* cancellable,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN gchar** zeitgeist_log_find_related_uris_finish (ZeitgeistLog* self,
                                                GAsyncResult* _res_,
                                                gint* result_length1,
                                                GError** error);
VALA_EXTERN void zeitgeist_log_delete_events (ZeitgeistLog* self,
                                  GArray* event_ids,
                                  GCancellable* cancellable,
                                  GAsyncReadyCallback _callback_,
                                  gpointer _user_data_);
VALA_EXTERN ZeitgeistTimeRange* zeitgeist_log_delete_events_finish (ZeitgeistLog* self,
                                                        GAsyncResult* _res_,
                                                        GError** error);
VALA_EXTERN void zeitgeist_log_quit (ZeitgeistLog* self,
                         GCancellable* cancellable,
                         GAsyncReadyCallback _callback_,
                         gpointer _user_data_);
VALA_EXTERN void zeitgeist_log_quit_finish (ZeitgeistLog* self,
                                GAsyncResult* _res_,
                                GError** error);
VALA_EXTERN GType zeitgeist_monitor_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistMonitor, g_object_unref)
VALA_EXTERN void zeitgeist_log_install_monitor (ZeitgeistLog* self,
                                    ZeitgeistMonitor* monitor,
                                    GError** error);
VALA_EXTERN void zeitgeist_log_remove_monitor (ZeitgeistLog* self,
                                   ZeitgeistMonitor* monitor,
                                   GError** error);
VALA_EXTERN void zeitgeist_log_get_version (ZeitgeistLog* self,
                                gint* major,
                                gint* minor,
                                gint* micro);
VALA_EXTERN gchar** zeitgeist_log_get_extensions (ZeitgeistLog* self,
                                      gint* result_length1);
VALA_EXTERN gchar* zeitgeist_log_datapath (ZeitgeistLog* self);
VALA_EXTERN GType zeitgeist_remote_monitor_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint zeitgeist_remote_monitor_register_object (void* object,
                                                GDBusConnection* connection,
                                                const gchar* path,
                                                GError** error);
VALA_EXTERN GType zeitgeist_remote_monitor_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistRemoteMonitor, g_object_unref)
VALA_EXTERN ZeitgeistMonitor* zeitgeist_monitor_new (ZeitgeistTimeRange* time_range,
                                         GPtrArray* event_templates);
VALA_EXTERN ZeitgeistMonitor* zeitgeist_monitor_construct (GType object_type,
                                               ZeitgeistTimeRange* time_range,
                                               GPtrArray* event_templates);
VALA_EXTERN GPtrArray* zeitgeist_monitor_get_templates (ZeitgeistMonitor* self);
VALA_EXTERN char* zeitgeist_monitor_get_path (ZeitgeistMonitor* self);
VALA_EXTERN ZeitgeistTimeRange* zeitgeist_monitor_get_time_range (ZeitgeistMonitor* self);
VALA_EXTERN void zeitgeist_monitor_set_time_range (ZeitgeistMonitor* self,
                                       ZeitgeistTimeRange* value);
VALA_EXTERN GPtrArray* zeitgeist_monitor_get_event_templates (ZeitgeistMonitor* self);
VALA_EXTERN void zeitgeist_monitor_set_event_templates (ZeitgeistMonitor* self,
                                            GPtrArray* value);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_proxy_acquired (ZeitgeistQueuedProxyWrapper* self,
                                                    GObject* proxy);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_proxy_unavailable (ZeitgeistQueuedProxyWrapper* self,
                                                       GError* err);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_process_queued_methods (ZeitgeistQueuedProxyWrapper* self);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_name_owner_changed (ZeitgeistQueuedProxyWrapper* self,
                                                        GParamSpec* pspec);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_on_connection_established (ZeitgeistQueuedProxyWrapper* self);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_on_connection_lost (ZeitgeistQueuedProxyWrapper* self);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_wait_for_proxy (ZeitgeistQueuedProxyWrapper* self,
                                                    GAsyncReadyCallback _callback_,
                                                    gpointer _user_data_);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_wait_for_proxy_finish (ZeitgeistQueuedProxyWrapper* self,
                                                           GAsyncResult* _res_,
                                                           GError** error);
VALA_EXTERN ZeitgeistQueuedProxyWrapper* zeitgeist_queued_proxy_wrapper_construct (GType object_type);
VALA_EXTERN gboolean zeitgeist_queued_proxy_wrapper_get_proxy_created (ZeitgeistQueuedProxyWrapper* self);
VALA_EXTERN gboolean zeitgeist_queued_proxy_wrapper_get_is_connected (ZeitgeistQueuedProxyWrapper* self);
VALA_EXTERN gpointer zeitgeist_queued_proxy_wrapper_queued_method_ref (gpointer instance);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_queued_method_unref (gpointer instance);
VALA_EXTERN GParamSpec* zeitgeist_queued_proxy_wrapper_param_spec_queued_method (const gchar* name,
                                                                     const gchar* nick,
                                                                     const gchar* blurb,
                                                                     GType object_type,
                                                                     GParamFlags flags);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_value_set_queued_method (GValue* value,
                                                             gpointer v_object);
VALA_EXTERN void zeitgeist_queued_proxy_wrapper_value_take_queued_method (GValue* value,
                                                              gpointer v_object);
VALA_EXTERN gpointer zeitgeist_queued_proxy_wrapper_value_get_queued_method (const GValue* value);
VALA_EXTERN GType zeitgeist_queued_proxy_wrapper_queued_method_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistQueuedProxyWrapperQueuedMethod, zeitgeist_queued_proxy_wrapper_queued_method_unref)
VALA_EXTERN ZeitgeistQueuedProxyWrapperQueuedMethod* zeitgeist_queued_proxy_wrapper_queued_method_new (GSourceFunc callback,
                                                                                           gpointer callback_target,
                                                                                           GDestroyNotify callback_target_destroy_notify);
VALA_EXTERN ZeitgeistQueuedProxyWrapperQueuedMethod* zeitgeist_queued_proxy_wrapper_queued_method_construct (GType object_type,
                                                                                                 GSourceFunc callback,
                                                                                                 gpointer callback_target,
                                                                                                 GDestroyNotify callback_target_destroy_notify);
VALA_EXTERN GSourceFunc zeitgeist_queued_proxy_wrapper_queued_method_get_queued_method (ZeitgeistQueuedProxyWrapperQueuedMethod* self,
                                                                            gpointer* result_target);
VALA_EXTERN GType zeitgeist_version_struct_get_type (void) G_GNUC_CONST ;
VALA_EXTERN ZeitgeistVersionStruct* zeitgeist_version_struct_dup (const ZeitgeistVersionStruct* self);
VALA_EXTERN void zeitgeist_version_struct_free (ZeitgeistVersionStruct* self);
VALA_EXTERN GType zeitgeist_remote_log_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint zeitgeist_remote_log_register_object (void* object,
                                            GDBusConnection* connection,
                                            const gchar* path,
                                            GError** error);
VALA_EXTERN GType zeitgeist_remote_log_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistRemoteLog, g_object_unref)
VALA_EXTERN void zeitgeist_remote_log_delete_events (ZeitgeistRemoteLog* self,
                                         guint32* event_ids,
                                         gint event_ids_length1,
                                         GCancellable* cancellable,
                                         const char* sender,
                                         GAsyncReadyCallback _callback_,
                                         gpointer _user_data_);
VALA_EXTERN GVariant* zeitgeist_remote_log_delete_events_finish (ZeitgeistRemoteLog* self,
                                                     GAsyncResult* _res_,
                                                     GError** error);
VALA_EXTERN void zeitgeist_remote_log_find_event_ids (ZeitgeistRemoteLog* self,
                                          GVariant* time_range,
                                          GVariant* event_templates,
                                          guint storage_state,
                                          guint num_events,
                                          guint result_type,
                                          GCancellable* cancellable,
                                          const char* sender,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN guint32* zeitgeist_remote_log_find_event_ids_finish (ZeitgeistRemoteLog* self,
                                                     GAsyncResult* _res_,
                                                     gint* result_length1,
                                                     GError** error);
VALA_EXTERN void zeitgeist_remote_log_find_events (ZeitgeistRemoteLog* self,
                                       GVariant* time_range,
                                       GVariant* event_templates,
                                       guint storage_state,
                                       guint num_events,
                                       guint result_type,
                                       GCancellable* cancellable,
                                       const char* sender,
                                       GAsyncReadyCallback _callback_,
                                       gpointer _user_data_);
VALA_EXTERN GVariant* zeitgeist_remote_log_find_events_finish (ZeitgeistRemoteLog* self,
                                                   GAsyncResult* _res_,
                                                   GError** error);
VALA_EXTERN void zeitgeist_remote_log_find_related_uris (ZeitgeistRemoteLog* self,
                                             GVariant* time_range,
                                             GVariant* event_templates,
                                             GVariant* result_event_templates,
                                             guint storage_state,
                                             guint num_events,
                                             guint result_type,
                                             GCancellable* cancellable,
                                             const char* sender,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN gchar** zeitgeist_remote_log_find_related_uris_finish (ZeitgeistRemoteLog* self,
                                                       GAsyncResult* _res_,
                                                       gint* result_length1,
                                                       GError** error);
VALA_EXTERN void zeitgeist_remote_log_get_events (ZeitgeistRemoteLog* self,
                                      guint32* event_ids,
                                      gint event_ids_length1,
                                      GCancellable* cancellable,
                                      const char* sender,
                                      GAsyncReadyCallback _callback_,
                                      gpointer _user_data_);
VALA_EXTERN GVariant* zeitgeist_remote_log_get_events_finish (ZeitgeistRemoteLog* self,
                                                  GAsyncResult* _res_,
                                                  GError** error);
VALA_EXTERN void zeitgeist_remote_log_insert_events (ZeitgeistRemoteLog* self,
                                         GVariant* events,
                                         GCancellable* cancellable,
                                         const char* sender,
                                         GAsyncReadyCallback _callback_,
                                         gpointer _user_data_);
VALA_EXTERN guint32* zeitgeist_remote_log_insert_events_finish (ZeitgeistRemoteLog* self,
                                                    GAsyncResult* _res_,
                                                    gint* result_length1,
                                                    GError** error);
VALA_EXTERN void zeitgeist_remote_log_install_monitor (ZeitgeistRemoteLog* self,
                                           const char* monitor_path,
                                           GVariant* time_range,
                                           GVariant* event_templates,
                                           const char* owner,
                                           GAsyncReadyCallback _callback_,
                                           gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_log_install_monitor_finish (ZeitgeistRemoteLog* self,
                                                  GAsyncResult* _res_,
                                                  GError** error);
VALA_EXTERN void zeitgeist_remote_log_remove_monitor (ZeitgeistRemoteLog* self,
                                          const char* monitor_path,
                                          const char* owner,
                                          GAsyncReadyCallback _callback_,
                                          gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_log_remove_monitor_finish (ZeitgeistRemoteLog* self,
                                                 GAsyncResult* _res_,
                                                 GError** error);
VALA_EXTERN void zeitgeist_remote_log_quit (ZeitgeistRemoteLog* self,
                                GCancellable* cancellable,
                                GAsyncReadyCallback _callback_,
                                gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_log_quit_finish (ZeitgeistRemoteLog* self,
                                       GAsyncResult* _res_,
                                       GError** error);
VALA_EXTERN gchar** zeitgeist_remote_log_get_extensions (ZeitgeistRemoteLog* self,
                                             gint* result_length1);
VALA_EXTERN void zeitgeist_remote_log_get_version (ZeitgeistRemoteLog* self,
                                       ZeitgeistVersionStruct * result);
VALA_EXTERN gchar* zeitgeist_remote_log_get_datapath (ZeitgeistRemoteLog* self);
VALA_EXTERN void zeitgeist_remote_monitor_notify_insert (ZeitgeistRemoteMonitor* self,
                                             GVariant* time_range,
                                             GVariant* events,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_monitor_notify_insert_finish (ZeitgeistRemoteMonitor* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
VALA_EXTERN void zeitgeist_remote_monitor_notify_delete (ZeitgeistRemoteMonitor* self,
                                             GVariant* time_range,
                                             guint32* event_ids,
                                             gint event_ids_length1,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_monitor_notify_delete_finish (ZeitgeistRemoteMonitor* self,
                                                    GAsyncResult* _res_,
                                                    GError** error);
VALA_EXTERN GType zeitgeist_remote_simple_indexer_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint zeitgeist_remote_simple_indexer_register_object (void* object,
                                                       GDBusConnection* connection,
                                                       const gchar* path,
                                                       GError** error);
VALA_EXTERN GType zeitgeist_remote_simple_indexer_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistRemoteSimpleIndexer, g_object_unref)
VALA_EXTERN void zeitgeist_remote_simple_indexer_search (ZeitgeistRemoteSimpleIndexer* self,
                                             const gchar* query_string,
                                             GVariant* time_range,
                                             GVariant* filter_templates,
                                             guint offset,
                                             guint count,
                                             guint result_type,
                                             GCancellable* cancellable,
                                             GAsyncReadyCallback _callback_,
                                             gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_simple_indexer_search_finish (ZeitgeistRemoteSimpleIndexer* self,
                                                    GAsyncResult* _res_,
                                                    GVariant** events,
                                                    guint* matches,
                                                    GError** error);
VALA_EXTERN void zeitgeist_remote_simple_indexer_search_with_relevancies (ZeitgeistRemoteSimpleIndexer* self,
                                                              const gchar* query_string,
                                                              GVariant* time_range,
                                                              GVariant* filter_templates,
                                                              guint storage_state,
                                                              guint offset,
                                                              guint count,
                                                              guint result_type,
                                                              GCancellable* cancellable,
                                                              GAsyncReadyCallback _callback_,
                                                              gpointer _user_data_);
VALA_EXTERN void zeitgeist_remote_simple_indexer_search_with_relevancies_finish (ZeitgeistRemoteSimpleIndexer* self,
                                                                     GAsyncResult* _res_,
                                                                     GVariant** events,
                                                                     gdouble** relevancies,
                                                                     gint* relevancies_length1,
                                                                     guint* matches,
                                                                     GError** error);
VALA_EXTERN GType zeitgeist_network_manager_dbus_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint zeitgeist_network_manager_dbus_register_object (void* object,
                                                      GDBusConnection* connection,
                                                      const gchar* path,
                                                      GError** error);
VALA_EXTERN GType zeitgeist_network_manager_dbus_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistNetworkManagerDBus, g_object_unref)
VALA_EXTERN guint32 zeitgeist_network_manager_dbus_state (ZeitgeistNetworkManagerDBus* self,
                                              GError** error);
VALA_EXTERN GType zeitgeist_connman_manager_dbus_proxy_get_type (void) G_GNUC_CONST ;
VALA_EXTERN guint zeitgeist_connman_manager_dbus_register_object (void* object,
                                                      GDBusConnection* connection,
                                                      const gchar* path,
                                                      GError** error);
VALA_EXTERN GType zeitgeist_connman_manager_dbus_get_type (void) G_GNUC_CONST ;
G_DEFINE_AUTOPTR_CLEANUP_FUNC (ZeitgeistConnmanManagerDBus, g_object_unref)
VALA_EXTERN gchar* zeitgeist_connman_manager_dbus_get_state (ZeitgeistConnmanManagerDBus* self,
                                                 GError** error);

G_END_DECLS

#endif
